require 'socket'

module Mongoid
  module Lock
    
    def reset_lock!
      self.update_attributes!({:sema_count => __semaphore_initial_count, :used_by => nil})
    end
    
    def lock_acquire
      self.sema_down
      if (self.sema_ready?)
        self.set(:used_by, "#{Socket.hostname}:#{Process.pid}")
      else
        self.sema_up
        raise Mongoid::Semaphore::UnsynchronizedAccess.new(self.sema_count, "Lock in Use! (#{self.used_by})")
      end
    end
    
    def lock_release
      if (self.used_by == "#{Socket.hostname}:#{Process.pid}")
        self.sema_up
        self.set(:used_by, nil)
      end
    end
    
    
    def synchronized(&block)
      self.lock_acquire
      begin
        block.call()
      ensure
        self.lock_release
      end
    end
    
    def try_synchronized(&try_block)
      return false unless self.sema_ready?
      begin
        self.synchronized do
          try_block.call()
        end
        return true
      rescue Mongoid::Semaphore::UnsynchronizedAccess => ua
        return false
      end
    end
  end
end