require 'mongoid'
require 'mongoid/semaphore/class_methods'
require 'mongoid/semaphore/sync_methods'

module Mongoid
  module Semaphore
#------------------------------------------------------------------------------
# => Embedded class for Unsynchronized use Exceptions
#------------------------------------------------------------------------------
    class UnsynchronizedAccess < StandardError
      attr_reader :sema_count
      def initialize(count, *args)
        @sema_count = count
        super(*args)
      end
    end
#------------------------------------------------------------------------------
# => Overridden Instance Attribute Accessors on Volatile Fields
#------------------------------------------------------------------------------
    # => Volatile Value, Refresh Before Returning
    def sema_count
      self.save()
      self.reload
      return super
    end
#------------------------------------------------------------------------------
# => Inclusion hook. Adds fields, and default attributes
#------------------------------------------------------------------------------    
    def self.included(base)
      super if defined?(super)
      base.class_eval do
        # => Define necessary fields for synchronization
        unless base.included_modules.include?(Mongoid::Document)
          include Mongoid::Document
        end
        base.extend(Mongoid::Semaphore::ClassMethods)
        # =>  Semaphore Counter
        field :sema_count, :type => Integer, :default => lambda { self.class.__semaphore_initial_count }
      end
    end
  end
end

