module Mongoid
  module Semaphore
    def reset_semaphore!
      self.update_attributes!({:sema_count => __semaphore_initial_count})
    end
    
    def sema_up
      self.inc(:sema_count, 1)
    end
    
    def sema_down
      self.dec(:sema_count, 1)
    end
    
    def sema_ready?
      self.save()
      self.reload()
      return (self.sema_count >= 0)
    end
    
    def synchronized(&block)
      begin
        self.sema_down
        if (self.sema_ready?)
          block.call()
        else
          raise Mongoid::Semaphore::UnsynchronizedAccess.new(self.sema_count)
        end
      ensure
        self.sema_up
      end
    end
    
    def try_synchronized(&try_block)
      return false unless self.sema_ready?
      begin
        self.synchronized do
          try_block.call()
        end
        return true
      rescue Mongoid::Semaphore::UnsynchronizedAccess => ua
        return false
      end
    end
  end
end